<?php
// $Id: image_gallery_access.admin.inc,v 1.2 2009/01/27 00:50:20 salvis Exp $

/**
 * @file image_gallery_access.admin.inc
 *
 * Include file for image_gallery_access.module, containing (sub-)page handling
 * (form_alter) and batch code.
 *
 */

/**
 * Rewrite the image gallery administration page with our new access rules.
 */
function _image_gallery_access_admin_form(&$form, &$form_state, $form_id) {
  $rids = array();
  $result = db_query("SELECT r.rid, r.name FROM {role} r ORDER BY r.name");
  while ($obj = db_fetch_object($result)) {
    $rids[$obj->rid] = $obj->name;
  }

  if (isset($form['tid']['#value'])) {  // edit
    $result = db_query("SELECT * FROM {image_gallery_access} where tid=%d", $form['tid']['#value']);

    $view = $update = $delete = $create = array();
    while ($image_gallery_access = db_fetch_object($result)) {
      $row_received = TRUE;
      if ($image_gallery_access->grant_view) {
        $view[] = $image_gallery_access->rid;
      }
      if ($image_gallery_access->grant_update) {
        $update[] = $image_gallery_access->rid;
      }
      if ($image_gallery_access->grant_delete) {
        $delete[] = $image_gallery_access->rid;
      }
      if ($image_gallery_access->grant_create) {
        $create[] = $image_gallery_access->rid;
      }
      if ($image_gallery_access->rid == DRUPAL_AUTHENTICATED_RID) {  // this is our reference
        $iga_priority = $image_gallery_access->priority;
      }
    }
    if (!isset($row_received) && empty($form['#parameters'][1]['#post'])) {
      drupal_set_message(t('If you have only just installed !Image_Gallery_Access, then the images in this gallery may still be accessible, but once your permissions get rebuilt (intentionally or behind-the-scenes by some other module), the images in this gallery will vanish &mdash; so, be sure to set the desired Access Control below and save!', array('!Image_Gallery_Access' => 'Image Gallery Access')), 'error');
    }
  }
  else {  // create
    // Default to all users can read; all logged in users can post.
    $view = array(DRUPAL_ANONYMOUS_RID, DRUPAL_AUTHENTICATED_RID);
    $create = array(DRUPAL_AUTHENTICATED_RID);
    $update = $delete = array();
  }

  $form['image_gallery_access'] = array('#type' => 'fieldset',
    '#title' => t('Access control'),
    '#collapsible' => TRUE,
    '#tree' => TRUE,
  );

  $tr = 't';
  $variables = array(
    '!access_content' => '<em>'. l($tr('access content'), 'admin/user/permissions', array('fragment' => 'module-node', 'html' => TRUE)). '</em>', 
    '%administer_nodes' => $tr('administer nodes')
  );
  $form['image_gallery_access']['note'] = array('#type' => 'markup',
    '#value' => '<div>'. t('Note that users need at least the !access_content permission to be able to deal in any way with content. Furthermore note that content which is not published is treated in a different way by Drupal: It can be viewed only by its author or users with the %administer_nodes permission.', $variables) .'</div>',
  );
  $form['image_gallery_access']['view'] = array('#type' => 'checkboxes',
    '#prefix' => '<div class="image-gallery-access-div">',
    '#suffix' => '</div>',
    '#options' => $rids,
    '#title' => t('View this gallery'),
    '#default_value' => $view
  );

  $form['image_gallery_access']['create'] = array('#type' => 'checkboxes',
    '#prefix' => '<div class="image-gallery-access-div">',
    '#suffix' => '</div>',
    '#options' => $rids,
    '#title' => t('Post in this gallery'),
    '#default_value' => $create
  );
  $form['image_gallery_access']['update'] = array('#type' => 'checkboxes',
    '#prefix' => '<div class="image-gallery-access-div">',
    '#suffix' => '</div>',
    '#options' => $rids,
    '#title' => t('Edit images'),
    '#default_value' => $update
  );
  $form['image_gallery_access']['delete'] = array('#type' => 'checkboxes',
    '#prefix' => '<div class="image-gallery-access-div">',
    '#suffix' => '</div>',
    '#options' => $rids,
    '#title' => t('Delete images'),
    '#default_value' => $delete
  );
  // Find our moderator ACL:

  $form['image_gallery_access']['clearer'] = array(
    '#value' => '<div class="image-gallery-access-clearer"></div>',
  );
  drupal_add_css(drupal_get_path('module', 'image_gallery_access') .'/image_gallery_access.css');
  if (isset($form['tid']['#value'])) {  // edit, not new
    $acl_id = db_result(db_query("SELECT acl_id from {acl} WHERE module = 'image_gallery_access' AND name = '%d'", $form['tid']['#value']));
    if (!$acl_id) { // create one
      $acl_id = acl_create_new_acl('image_gallery_access', $form['tid']['#value']);
      // update every existing node in this image gallery to use this acl.
      $result = db_query("SELECT nid FROM {term_node} WHERE tid = %d", $form['tid']['#value']);
      while ($node = db_fetch_object($result)) {
        // all privs to this ACL.
        acl_node_add_acl($node->nid, $acl_id, 1, 1, 1);
      }
    }
    $form['image_gallery_access']['acl'] = acl_edit_form($acl_id, 'Moderators');
    if (strlen($form['image_gallery_access']['acl']['user_list']['#default_value']) <= 6 && empty($form_state['post']['image_gallery_access']['acl']['add']))
    {
      $form['image_gallery_access']['acl']['#collapsed'] = TRUE;
    }
    $form['image_gallery_access']['acl']['note'] = array(
      '#type' => 'markup',
      '#value' => '<div>'. t('Note: Changes to moderators are not saved until you click [!Submit] below.', array('!Submit' => $tr('Submit'))) .'</div>',
    );
  }

  foreach (module_implements('node_access_records') as $module) {
    $na_modules[$module] = $module;
  }
  unset($na_modules['image_gallery_access']);
  unset($na_modules['acl']);
  if (count($na_modules)) {
    $form['image_gallery_access']['interference'] = array(
      '#type' => 'fieldset',
      '#title' => t('Module interference'),
      '#collapsible' => TRUE,
    );
    $tr = 't';
    $variables = array(
      '%content_type' => node_get_types('name', 'image'),
      '!Image_Gallery_Access' => 'Image Gallery Access',
      '!Content_Access' => 'Content Access',
      '!ACL' => 'ACL',
      '!module_list' => '<ul><li>'. implode($na_modules, '</li><li>') .'</li></ul>',
      '%devel_node_access' => 'devel_node_access',
      '!Devel' => l('Devel', 'http://drupal.org/project/devel'),
      '!debug_mode' => l('debug mode', 'admin/settings/devel', array('fragment' => 'edit-devel-node-access-debug-mode')),
    );
    $form['image_gallery_access']['interference'][] = array(
      '#type' => 'item',
      '#value' => '<p>'. t("Besides !Image_Gallery_Access (and !ACL) you have installed the following node access module(s): !module_list   The grants of every module will be combined for each node. Access can only be granted, not removed&mdash;if a certain module grants a permission, the other(s) cannot deny it.", $variables) .'</p>',
      '#description' => t('In case of problems, you should temporarily (!) install the %devel_node_access module (part of the !Devel module) and enable its !debug_mode: this will show you all the grants that actually control your nodes.', $variables),
    );

    if (module_exists('content_access')) {
      $ca_settings = variable_get('content_access_settings', array());
      foreach (array('view', 'update', 'delete', 'per_node') as $type) {
        $value = content_access_get_settings($type, 'image');
        if (!empty($value)) {
          $ca_interferes = TRUE;
        }
      }
      $ca_priority = content_access_get_settings('priority', 'image');
      $is_conflict = $ca_priority >= $iga_priority && !empty($ca_interferes) || $ca_priority > $iga_priority;
      $variables += array(
        '!Content_Access' => 'Content Access',
        '!link' => l(t('!Content_Access configuration for the %content_type type', $variables), 'admin/content/node-type/image/access', array('html' => TRUE)),
        '%Advanced' => $tr('Advanced'),
      );
      $specifically = ($ca_priority == $iga_priority ? t('Specifically, any grants given by !Content_Access cannot be taken back by !Image_Gallery_Access.', $variables) : '');
      if ($is_conflict) {
        $form['image_gallery_access']['interference']['by_content_access'] = array(
          '#type' => 'fieldset',
          '#title' => 'Content Access',
          '#collapsible' => FALSE,
          '#attributes' => array('class' => 'error'),
        );
        $form['image_gallery_access']['interference']['by_content_access'][] = array(
          '#value' => '<div>'. t('You have set the !Content_Access module to control access to content of type %content_type&mdash;this can interfere with proper operation of !Image_Gallery_Access!', $variables) ." $specifically</div>",
        );
        if ($ca_priority == $iga_priority) {
          $form['image_gallery_access']['interference']['by_content_access'][] = array(
            '#value' => '<div>'. t("Unless you really know what you're doing, we recommend that you go to the !link page and clear all checkboxes. This will instruct !Content_Access to leave the %content_type nodes alone.", $variables) .'</div>',
          );
        }
        else {
          $form['image_gallery_access']['interference']['by_content_access'][] = array(
            '#value' => '<div>'. t("The priority of !Content_Access ($ca_priority) is higher than the priority of !Image_Gallery_Access ($iga_priority), which means the latter is <b>completely disabled</b> for the %content_type type! Unless you really know what you're doing, we recommend that you go to the !link page, change the priority (under %Advanced) to 0, and clear all checkboxes.", $variables) .'</div>',
          );
        }
      }
      else {
        $form['image_gallery_access']['interference'][] = array(
          '#value' => '<p>'. t('Note: You have installed the !Content_Access module, which has the capability to grant access to content that would otherwise be protected by !Image_Gallery_Access. Be careful when configuring !Content_Access!', $variables) .'</p>',
        );
      }
    }

    $form['image_gallery_access']['interference']['advanced'] = array(
      '#type' => 'fieldset',
      '#title' => t('Advanced'),
      '#collapsible' => TRUE,
      '#collapsed' => TRUE,
    );
    $form['image_gallery_access']['interference']['advanced']['priority'] = array(
      '#type' => 'weight',
      '#title' => t('Priority of !Image_Gallery_Access node grants in this gallery', $variables),
      '#default_value' => $iga_priority,
      '#description' => t("If you have no other node access control modules installed, you should leave this at the default 0. <br /> Otherwise you can raise or lower the priority of !Image_Gallery_Access' grants. Out of all the grants contributed to a node, only those with the highest priority are used, and all others are discarded.", $variables),
    );
  }

  // Move some stuff down so our block goes in a nice place.
  $form['submit']['#weight'] = 10;
  $form['delete']['#weight'] = 10;

  $form['#submit'][] = '_image_gallery_access_form_submit';
}

function _image_gallery_access_form_submit($form, &$form_state) {
  db_query("DELETE FROM {image_gallery_access} WHERE tid = %d", $form_state['values']['tid']);
  $access = $form_state['values']['image_gallery_access']; // shortcut
  $iga_priority = $access['interference']['advanced']['priority'];
  if (array_key_exists('acl', $access)) {
    acl_save_form($access['acl'], $iga_priority);
  }
  foreach ($access['view'] as $rid => $checked) {
    db_query("INSERT INTO {image_gallery_access} (tid, rid, grant_view, grant_update, grant_delete, grant_create, priority) VALUES (%d, %d, %d, %d, %d, %d, %d)", $form_state['values']['tid'], $rid, (bool) $checked, !empty($access['update'][$rid]), !empty($access['delete'][$rid]), !empty($access['create'][$rid]), $iga_priority);
  }

  // mass update in batch mode, modeled after node.module
  $batch = array(
    'title' => t('Updating content access permissions'),
    'file' => drupal_get_path('module', 'image_gallery_access') .'/image_gallery_access.admin.inc',
    'operations' => array(
      array('_image_gallery_access_update_batch_operation', array($form_state['values']['tid'])),
    ),
    'finished' => '_image_gallery_access_update_batch_finished'
  );
  batch_set($batch);
}

/**
 * Batch operation for image_gallery_access_form_submit().
 *
 * This is a mutlistep operation : we go through all nodes by packs of 20.
 * The batch processing engine interrupts processing and sends progress
 * feedback after 1 second execution time.
 */
function _image_gallery_access_update_batch_operation($tid, &$context) {
  if (empty($context['sandbox'])) {
    // Initiate multistep processing.
    $context['sandbox']['progress'] = 0;
    $context['sandbox']['current_node'] = 0;
    $context['sandbox']['max'] = db_result(db_query("SELECT COUNT(DISTINCT n.nid) FROM {node} n LEFT JOIN {term_node} tn ON tn.nid = n.nid WHERE tn.tid = %d", $tid));
  }

  // Process the next 20 nodes.
  $limit = 20;
  $result = db_query_range("SELECT n.nid FROM {node} n LEFT JOIN {term_node} tn ON tn.nid = n.nid WHERE n.nid > %d AND tn.tid = %d ORDER BY n.nid ASC", $context['sandbox']['current_node'], $tid, 0, $limit);
  while ($row = db_fetch_array($result)) {
    $loaded_node = node_load($row['nid'], NULL, TRUE);
    // To preserve database integrity, only aquire grants if the node
    // loads successfully.
    if (!empty($loaded_node)) {
      node_access_acquire_grants($loaded_node);
    }
    $context['sandbox']['progress']++;
    $context['sandbox']['current_node'] = $loaded_node->nid;
  }

  // Multistep processing : report progress.
  if ($context['sandbox']['progress'] != $context['sandbox']['max']) {
    $context['finished'] = $context['sandbox']['progress'] / $context['sandbox']['max'];
  }
}

/**
 * Post-processing for image_gallery_access_form_submit().
 */
function _image_gallery_access_update_batch_finished($success, $results, $operations) {
  if ($success) {
    drupal_set_message(t('The content access permissions have been updated.'));
  }
  else {
    drupal_set_message(t('The content access permissions have not been properly updated.'), 'error');
    node_access_needs_rebuild(TRUE);
  }
  cache_clear_all();
}

/**
 * Add warnings on Content Access admin forms where CA wants
 * to control the same content types as we do.
 */
function _image_gallery_access_content_access_admin_form() {
  $tr = 't';
  $variables = array(
    '!content_type' => $tr('content type'),
    '!Image_Gallery_Access' => 'Image Gallery Access',
    '!Image_Gallery_Access_link' => l('Image Gallery Access', 'admin/content/image')
  );
  drupal_set_message(t('Note: In Drupal, access can only be granted, not taken away. Whatever access you grant here will not be reflected on the !Image_Gallery_Access_link settings, but !Image_Gallery_Access can only allow <i>more</i> access, not less.', $variables), 'warning');
}
